@@ -19,15 +19,18 @@
 #' Frequency table
 #'
 #' Create a frequency table of a vector with items or a data frame. Supports quasiquotation and markdown for reports. \code{top_freq} can be used to get the top/bottom \emph{n} items of a frequency table, with counts as names.
-#' @param x vector of any class or a \code{\link{data.frame}}, \code{\link{tibble}} or \code{\link{table}}
+#' @param x vector of any class or a \code{\link{data.frame}}, \code{\link{tibble}} (may contain a grouping variable) or \code{\link{table}}
 #' @param ... up to nine different columns of \code{x} when \code{x} is a \code{data.frame} or \code{tibble}, to calculate frequencies from - see Examples
-#' @param sort.count sort on count, i.e. frequencies. This will be \code{TRUE} at default for everything except for factors.
+#' @param sort.count sort on count, i.e. frequencies. This will be \code{TRUE} at default for everything except when using grouping variables.
 #' @param nmax number of row to print. The default, \code{15}, uses \code{\link{getOption}("max.print.freq")}. Use \code{nmax = 0}, \code{nmax = Inf}, \code{nmax = NULL} or \code{nmax = NA} to print all rows.
-#' @param na.rm a logical value indicating whether \code{NA} values should be removed from the frequency table. The header will always print the amount of \code{NA}s.
+#' @param na.rm a logical value indicating whether \code{NA} values should be removed from the frequency table. The header (if set) will always print the amount of \code{NA}s.
 #' @param row.names a logical value indicating whether row indices should be printed as \code{1:nrow(x)}
-#' @param markdown print table in markdown format (this forces \code{nmax = NA})
+#' @param markdown a logical value indicating whether the frequency table should be printed in markdown format. This will print all rows and is default behaviour in non-interactive R sessions (like when knitting RMarkdown files).
 #' @param digits how many significant digits are to be used for numeric values in the header (not for the items themselves, that depends on \code{\link{getOption}("digits")})
 #' @param quote a logical value indicating whether or not strings should be printed with surrounding quotes
+#' @param header a logical value indicating whether an informative header should be printed
+#' @param title text to show above frequency table, at default to tries to coerce from the variables passed to \code{x}
+#' @param na a character string to should be used to show empty (\code{NA}) values (only useful when \code{na.rm = FALSE})
 #' @param sep a character string to separate the terms when selecting multiple columns
 #' @param f a frequency table
 #' @param n number of top \emph{n} items to return, use -n for the bottom \emph{n} items. It will include more than \code{n} rows if there are ties.
@@ -56,12 +59,14 @@
 #' The function \code{top_freq} uses \code{\link[dplyr]{top_n}} internally and will include more than \code{n} rows if there are ties.
 #' @importFrom stats fivenum sd mad
 #' @importFrom grDevices boxplot.stats
-#' @importFrom dplyr %>% select pull n_distinct group_by arrange desc mutate summarise n_distinct tibble
-#' @importFrom utils browseVignettes installed.packages
+#' @importFrom dplyr %>% arrange arrange_at desc filter_at funs group_by mutate mutate_at n_distinct pull select summarise tibble ungroup vars all_vars
+#' @importFrom utils browseVignettes
+#' @importFrom hms is.hms
+#' @importFrom crayon red green silver
 #' @keywords summary summarise frequency freq
 #' @rdname freq
 #' @name freq
-#' @return A \code{data.frame} with an additional class \code{"frequency_tbl"}
+#' @return A \code{data.frame} (with an additional class \code{"frequency_tbl"}) with five columns: \code{item}, \code{count}, \code{percent}, \code{cum_count} and \code{cum_percent}.
 #' @export
 #' @examples
 #' library(dplyr)
@@ -72,7 +77,8 @@
 #' septic_patients$hospital_id %>% freq()
 #' septic_patients[, "hospital_id"] %>% freq()
 #' septic_patients %>% freq("hospital_id")
-#' septic_patients %>% freq(hospital_id)  #<- easiest to remember when you're used to tidyverse
+#' septic_patients %>% freq(hospital_id)  #<- easiest to remember (tidyverse)
+#'
 #'
 #' # you could also use `select` or `pull` to get your variables
 #' septic_patients %>%
@@ -80,44 +86,59 @@
 #'   select(mo) %>%
 #'   freq()
 #'
+#'
 #' # multiple selected variables will be pasted together
 #' septic_patients %>%
 #'   left_join_microorganisms %>%
 #'   filter(hospital_id == "A") %>%
 #'   freq(genus, species)
 #'
+#'
+#' # group a variable and analyse another
+#' septic_patients %>%
+#'   group_by(hospital_id) %>%
+#'   freq(gender)
+#'
+#'
 #' # get top 10 bugs of hospital A as a vector
 #' septic_patients %>%
 #'   filter(hospital_id == "A") %>%
 #'   freq(mo) %>%
 #'   top_freq(10)
 #'
+#'
 #' # save frequency table to an object
 #' years <- septic_patients %>%
 #'   mutate(year = format(date, "%Y")) %>%
 #'   freq(year)
 #'
+#'
 #' # show only the top 5
 #' years %>% print(nmax = 5)
 #'
+#'
 #' # save to an object with formatted percentages
 #' years <- format(years)
 #'
+#'
 #' # print a histogram of numeric values
 #' septic_patients %>%
 #'   freq(age) %>%
 #'   hist()
 #'
+#'
 #' # or print all points to a regular plot
 #' septic_patients %>%
 #'   freq(age) %>%
 #'   plot()
 #'
+#'
 #' # transform to a data.frame or tibble
 #' septic_patients %>%
 #'   freq(age) %>%
 #'   as.data.frame()
 #'
+#'
 #' # or transform (back) to a vector
 #' septic_patients %>%
 #'   freq(age) %>%
@@ -129,34 +150,43 @@
 #'             sort(),
 #'           sort(septic_patients$age)) # TRUE
 #'
-#' # it also supports `table` objects:
+#'
+#' # it also supports `table` objects
 #' table(septic_patients$gender,
 #'       septic_patients$age) %>%
 #'   freq(sep = " **sep** ")
 #'
+#'
+#' # only get selected columns
+#' septic_patients %>%
+#'   freq(hospital_id) %>%
+#'   select(item, percent)
+#'
+#' septic_patients %>%
+#'   freq(hospital_id) %>%
+#'   select(-count, -cum_count)
+#'
+#'
 #' # check differences between frequency tables
 #' diff(freq(septic_patients$trim),
 #'      freq(septic_patients$trsu))
-#'
-#' \dontrun{
-#' # send frequency table to clipboard (e.g. for pasting in Excel)
-#' septic_patients %>%
-#'   freq(age) %>%
-#'   format() %>%       # this will format the percentages
-#'   clipboard_export()
-#' }
 frequency_tbl <- function(x,
                           ...,
                           sort.count = TRUE,
                           nmax = getOption("max.print.freq"),
                           na.rm = TRUE,
                           row.names = TRUE,
-                          markdown = FALSE,
+                          markdown = !interactive(),
                           digits = 2,
                           quote = FALSE,
+                          header = !markdown,
+                          title = NULL,
+                          na = "<NA>",
                           sep = " ") {
 
   mult.columns <- 0
+  x.group = character(0)
+  df <- NULL
 
   x.name <- NULL
   cols <- NULL
@@ -174,6 +204,12 @@ frequency_tbl <- function(x,
   }
 
   if (any(class(x) == 'data.frame')) {
+    x.group <- group_vars(x)
+    if (length(x.group) > 1) {
+      x.group <- x.group[1L]
+      warning("freq supports one grouping variable, only `", x.group, "` will be kept.", call. = FALSE)
+    }
+
     if (is.null(x.name)) {
       x.name <- deparse(substitute(x))
     }
@@ -188,6 +224,30 @@ frequency_tbl <- function(x,
       if (!all(cols %in% colnames(x))) {
         stop("one or more columns not found: `", paste(cols, collapse = "`, `"), '`', call. = FALSE)
       }
+      if (length(x.group) > 0) {
+        x.group_cols <- c(x.group, cols)
+        df <- x %>%
+          group_by_at(vars(x.group_cols)) %>%
+          summarise(count = n())
+        if (na.rm == TRUE) {
+          df <- df %>% filter_at(vars(cols), all_vars(!is.na(.)))
+        }
+        if (!missing(sort.count)) {
+          if (sort.count == TRUE) {
+            df <- df %>% arrange_at(c(x.group, "count"), desc)
+          }
+        }
+        df <- df %>%
+          mutate(cum_count = cumsum(count))
+
+        df.topleft <- df[1, 1]
+        df <- df %>%
+          ungroup() %>%
+          # do not repeat group labels
+          mutate_at(vars(x.group), funs(ifelse(lag(.) == ., "", .)))
+        df[1, 1] <- df.topleft
+        colnames(df)[1:2] <- c("group", "item")
+      }
       if (length(cols) > 0) {
         x <- x[, cols]
       }
@@ -197,17 +257,14 @@ frequency_tbl <- function(x,
       cols <- NULL
     }
   } else if (any(class(x) == 'table')) {
-    if (!"tidyr" %in% rownames(installed.packages())) {
-      stop('transformation from `table` to frequency table requires the tidyr package.', call. = FALSE)
-    }
-    x <- x %>%
-      as.data.frame(stringsAsFactors = FALSE) %>%
-      # paste first two columns
-      tidyr::unite(col = "Pasted", 1:2, sep = sep, remove = TRUE)
-    x <- rep(x %>% pull(Pasted), x %>% pull(Freq))
+    x <- as.data.frame(x, stringsAsFactors = FALSE)
+    # now this DF contains 3 columns: the 2 vars and a Freq column
+    # paste the first 2 cols and repeat them Freq times:
+    x <- rep(x = do.call(paste, c(x[colnames(x)[1:2]], sep = sep)),
+             times = x$Freq)
     x.name <- "a `table` object"
     cols <- NULL
-    mult.columns <- 2
+    #mult.columns <- 2
   } else {
     x.name <- NULL
     cols <- NULL
@@ -217,74 +274,8 @@ frequency_tbl <- function(x,
     if (ncol(x) == 1 & any(class(x) == 'data.frame')) {
       x <- x %>% pull(1)
     } else if (ncol(x) < 10) {
-
       mult.columns <- ncol(x)
-
-      colnames(x) <- LETTERS[1:ncol(x)]
-      if (ncol(x) == 2) {
-        x$total <- paste(x$A %>% as.character(),
-                         x$B %>% as.character(),
-                         sep = sep)
-      } else if (ncol(x) == 3) {
-        x$total <- paste(x$A %>% as.character(),
-                         x$B %>% as.character(),
-                         x$C %>% as.character(),
-                         sep = sep)
-      } else if (ncol(x) == 4) {
-        x$total <- paste(x$A %>% as.character(),
-                         x$B %>% as.character(),
-                         x$C %>% as.character(),
-                         x$D %>% as.character(),
-                         sep = sep)
-      } else if (ncol(x) == 5) {
-        x$total <- paste(x$A %>% as.character(),
-                         x$B %>% as.character(),
-                         x$C %>% as.character(),
-                         x$D %>% as.character(),
-                         x$E %>% as.character(),
-                         sep = sep)
-      } else if (ncol(x) == 6) {
-        x$total <- paste(x$A %>% as.character(),
-                         x$B %>% as.character(),
-                         x$C %>% as.character(),
-                         x$D %>% as.character(),
-                         x$E %>% as.character(),
-                         x$F %>% as.character(),
-                         sep = sep)
-      } else if (ncol(x) == 7) {
-        x$total <- paste(x$A %>% as.character(),
-                         x$B %>% as.character(),
-                         x$C %>% as.character(),
-                         x$D %>% as.character(),
-                         x$E %>% as.character(),
-                         x$F %>% as.character(),
-                         x$G %>% as.character(),
-                         sep = sep)
-      } else if (ncol(x) == 8) {
-        x$total <- paste(x$A %>% as.character(),
-                         x$B %>% as.character(),
-                         x$C %>% as.character(),
-                         x$D %>% as.character(),
-                         x$E %>% as.character(),
-                         x$F %>% as.character(),
-                         x$G %>% as.character(),
-                         x$H %>% as.character(),
-                         sep = sep)
-      } else if (ncol(x) == 9) {
-        x$total <- paste(x$A %>% as.character(),
-                         x$B %>% as.character(),
-                         x$C %>% as.character(),
-                         x$D %>% as.character(),
-                         x$E %>% as.character(),
-                         x$F %>% as.character(),
-                         x$G %>% as.character(),
-                         x$H %>% as.character(),
-                         x$I %>% as.character(),
-                         sep = sep)
-      }
-
-      x <- x$total
-
+      x <- do.call(paste, c(x[colnames(x)], sep = sep))
     } else {
       stop('A maximum of 9 columns can be analysed at the same time.', call. = FALSE)
     }
@@ -302,12 +293,10 @@ frequency_tbl <- function(x,
     class(x) <- x_class
   }
 
-  if (missing(sort.count) & 'factor' %in% class(x)) {
-    # sort on factor level at default when x is a factor and sort.count is not set
-    sort.count <- FALSE
-  }
-
-  header <- character(0)
+  # if (sort.count == FALSE & 'factor' %in% class(x)) {
+  #   warning("Sorting a factor sorts on factor level, not necessarily alphabetically.", call. = FALSE)
+  # }
+  header_txt <- character(0)
 
   markdown_line <- ''
   if (markdown == TRUE) {
@@ -316,62 +305,82 @@ frequency_tbl <- function(x,
   x_align <- 'l'
 
   if (mult.columns > 0) {
-    header <- header %>% paste0(markdown_line, 'Columns:   ', mult.columns)
+    header_txt <- header_txt %>% paste0(markdown_line, 'Columns:   ', mult.columns)
   } else {
-    header <- header %>% paste0(markdown_line, 'Class:     ', class(x) %>% rev() %>% paste(collapse = " > "))
+    header_txt <- header_txt %>% paste0(markdown_line, 'Class:     ', class(x) %>% rev() %>% paste(collapse = " > "))
     if (!mode(x) %in% class(x)) {
-      header <- header %>% paste0(" (", mode(x), ")")
+      header_txt <- header_txt %>% paste0(silver(paste0(" (", mode(x), ")")))
     }
   }
 
-  header <- header %>% paste0(markdown_line, '\nLength:    ', (NAs %>% length() + x %>% length()) %>% format(),
-                              ' (of which NA: ', NAs %>% length() %>% format(),
-                              ' = ', (NAs %>% length() / (NAs %>% length() + x %>% length())) %>% percent(force_zero = TRUE, round = digits) %>% sub('NaN', '0', ., fixed = TRUE), ')')
-  header <- header %>% paste0(markdown_line, '\nUnique:    ', x %>% n_distinct() %>% format())
+  if (NROW(x) > 0) {
+    na_txt <- paste0(NAs %>% length() %>% format(), ' = ',
+                     (NAs %>% length() / (NAs %>% length() + x %>% length())) %>% percent(force_zero = TRUE, round = digits) %>%
+                       sub('NaN', '0', ., fixed = TRUE))
+    if (!na_txt %like% "^0 =") {
+      na_txt <- red(na_txt)
+    } else {
+      na_txt <- green(na_txt)
+    }
+    na_txt <- paste0('(of which NA: ', na_txt, ')')
+  } else {
+    na_txt <- ""
+  }
+
+  header_txt <- header_txt %>% paste0(markdown_line, '\nLength:    ', (NAs %>% length() + x %>% length()) %>% format(),
+                                      ' ', na_txt)
+  header_txt <- header_txt %>% paste0(markdown_line, '\nUnique:    ', x %>% n_distinct() %>% format())
 
   if (NROW(x) > 0 & any(class(x) == "character")) {
-    header <- header %>% paste0('\n')
-    header <- header %>% paste0(markdown_line, '\nShortest:  ', x %>% base::nchar() %>% base::min(na.rm = TRUE))
-    header <- header %>% paste0(markdown_line, '\nLongest:   ', x %>% base::nchar() %>% base::max(na.rm = TRUE))
+    header_txt <- header_txt %>% paste0('\n')
+    header_txt <- header_txt %>% paste0(markdown_line, '\nShortest:  ', x %>% base::nchar() %>% base::min(na.rm = TRUE))
+    header_txt <- header_txt %>% paste0(markdown_line, '\nLongest:   ', x %>% base::nchar() %>% base::max(na.rm = TRUE))
+  }
+
+  if (NROW(x) > 0 & any(class(x) == "difftime") & !is.hms(x)) {
+    header_txt <- header_txt %>% paste0('\n')
+    header_txt <- header_txt %>% paste(markdown_line, '\nUnits:    ', attributes(x)$units)
+    x <- as.double(x)
+    # after this, the numeric header_txt continues
   }
 
   if (NROW(x) > 0 & any(class(x) %in% c('double', 'integer', 'numeric', 'raw', 'single'))) {
     # right align number
     Tukey_five <- stats::fivenum(x, na.rm = TRUE)
     x_align <- 'r'
-    header <- header %>% paste0('\n')
-    header <- header %>% paste(markdown_line, '\nMean:     ', x %>% base::mean(na.rm = TRUE) %>% format(digits = digits))
-    header <- header %>% paste0(markdown_line, '\nStd. dev.: ', x %>% stats::sd(na.rm = TRUE) %>% format(digits = digits),
-                                ' (CV: ', x %>% cv(na.rm = TRUE) %>% format(digits = digits),
-                                ', MAD: ', x %>% stats::mad(na.rm = TRUE) %>% format(digits = digits), ')')
-    header <- header %>% paste0(markdown_line, '\nFive-Num:  ', Tukey_five %>% format(digits = digits) %>% trimws() %>% paste(collapse = ' | '),
-                                ' (IQR: ', (Tukey_five[4] - Tukey_five[2]) %>% format(digits = digits),
-                                ', CQV: ', x %>% cqv(na.rm = TRUE) %>% format(digits = digits), ')')
+    header_txt <- header_txt %>% paste0('\n')
+    header_txt <- header_txt %>% paste(markdown_line, '\nMean:     ', x %>% base::mean(na.rm = TRUE) %>% format(digits = digits))
+    header_txt <- header_txt %>% paste0(markdown_line, '\nStd. dev.: ', x %>% stats::sd(na.rm = TRUE) %>% format(digits = digits),
+                                        ' (CV: ', x %>% cv(na.rm = TRUE) %>% format(digits = digits),
+                                        ', MAD: ', x %>% stats::mad(na.rm = TRUE) %>% format(digits = digits), ')')
+    header_txt <- header_txt %>% paste0(markdown_line, '\nFive-Num:  ', Tukey_five %>% format(digits = digits) %>% trimws() %>% paste(collapse = ' | '),
+                                        ' (IQR: ', (Tukey_five[4] - Tukey_five[2]) %>% format(digits = digits),
+                                        ', CQV: ', x %>% cqv(na.rm = TRUE) %>% format(digits = digits), ')')
     outlier_length <- length(boxplot.stats(x)$out)
-    header <- header %>% paste0(markdown_line, '\nOutliers:  ', outlier_length)
+    header_txt <- header_txt %>% paste0(markdown_line, '\nOutliers:  ', outlier_length)
     if (outlier_length > 0) {
-      header <- header %>% paste0(' (unique: ', boxplot.stats(x)$out %>% n_distinct(), ')')
+      header_txt <- header_txt %>% paste0(' (unique count: ', boxplot.stats(x)$out %>% n_distinct(), ')')
     }
   }
   if (NROW(x) > 0 & any(class(x) == "rsi")) {
-    header <- header %>% paste0('\n')
-    cnt_S <- sum(x == "S")
-    cnt_I <- sum(x == "I")
-    cnt_R <- sum(x == "R")
-    header <- header %>% paste(markdown_line, '\n%IR:      ',
-                               ((cnt_I + cnt_R) / sum(!is.na(x))) %>% percent(force_zero = TRUE, round = digits))
-    header <- header %>% paste0(markdown_line, '\nRatio SIR: 1.0 : ',
-                                (cnt_I / cnt_S) %>% format(digits = 1, nsmall = 1), " : ",
-                                (cnt_R / cnt_S) %>% format(digits = 1, nsmall = 1))
+    header_txt <- header_txt %>% paste0('\n')
+    cnt_S <- sum(x == "S", na.rm = TRUE)
+    cnt_IR <- sum(x %in% c("I", "R"), na.rm = TRUE)
+    header_txt <- header_txt %>% paste(markdown_line, '\n%IR:      ',
+                                       (cnt_IR / sum(!is.na(x), na.rm = TRUE)) %>% percent(force_zero = TRUE, round = digits),
+                                       paste0('(ratio S : IR = 1.0 : ', (cnt_IR / cnt_S) %>% format(digits = 1, nsmall = 1), ")"))
+    if (NROW(x) < 30) {
+      header_txt <- header_txt %>% paste(markdown_line, red('\nToo few isolates for reliable resistance interpretation.'))
+    }
   }
 
   formatdates <- "%e %B %Y" # = d mmmm yyyy
-  if (any(class(x) == 'hms')) {
+  if (is.hms(x)) {
     x <- x %>% as.POSIXlt()
     formatdates <- "%H:%M:%S"
   }
   if (NROW(x) > 0 & any(class(x) %in% c('Date', 'POSIXct', 'POSIXlt'))) {
-    header <- header %>% paste0('\n')
+    header_txt <- header_txt %>% paste0('\n')
     mindate <- x %>% min(na.rm = TRUE)
     maxdate <- x %>% max(na.rm = TRUE)
     maxdate_days <- difftime(maxdate, mindate, units = 'auto') %>% as.double()
@@ -380,17 +389,17 @@ frequency_tbl <- function(x,
 
     if (formatdates == "%H:%M:%S") {
       # hms
-      header <- header %>% paste0(markdown_line, '\nEarliest:  ', mindate %>% format(formatdates) %>% trimws())
-      header <- header %>% paste0(markdown_line, '\nLatest:    ', maxdate %>% format(formatdates) %>% trimws(),
-                                  ' (+', difftime(maxdate, mindate, units = 'mins') %>% as.double() %>% format(digits = digits), ' min.)')
+      header_txt <- header_txt %>% paste0(markdown_line, '\nEarliest:  ', mindate %>% format(formatdates) %>% trimws())
+      header_txt <- header_txt %>% paste0(markdown_line, '\nLatest:    ', maxdate %>% format(formatdates) %>% trimws(),
+                                          ' (+', difftime(maxdate, mindate, units = 'mins') %>% as.double() %>% format(digits = digits), ' min.)')
     } else {
       # other date formats
-      header <- header %>% paste0(markdown_line, '\nOldest:    ', mindate %>% format(formatdates) %>% trimws())
-      header <- header %>% paste0(markdown_line, '\nNewest:    ', maxdate %>% format(formatdates) %>% trimws(),
-                                  ' (+', difftime(maxdate, mindate, units = 'auto') %>% as.double() %>% format(digits = digits), ')')
+      header_txt <- header_txt %>% paste0(markdown_line, '\nOldest:    ', mindate %>% format(formatdates) %>% trimws())
+      header_txt <- header_txt %>% paste0(markdown_line, '\nNewest:    ', maxdate %>% format(formatdates) %>% trimws(),
+                                          ' (+', difftime(maxdate, mindate, units = 'auto') %>% as.double() %>% format(digits = digits), ')')
     }
-    header <- header %>% paste0(markdown_line, '\nMedian:    ', mediandate %>% format(formatdates) %>% trimws(),
-                                ' (~', percent(median_days / maxdate_days, round = 0), ')')
+    header_txt <- header_txt %>% paste0(markdown_line, '\nMedian:    ', mediandate %>% format(formatdates) %>% trimws(),
+                                        ' (~', percent(median_days / maxdate_days, round = 0), ')')
   }
   if (any(class(x) == 'POSIXlt')) {
     x <- x %>% format(formatdates)
@@ -408,24 +417,27 @@ frequency_tbl <- function(x,
     nmax <- length(x)
   }
 
-  # create table with counts and percentages
-  column_names <- c('Item', 'Count', 'Percent', 'Cum. Count', 'Cum. Percent', '(Factor Level)')
-  column_names_df <- c('item', 'count', 'percent', 'cum_count', 'cum_percent', 'factor_level')
+  column_names <- c('Item', 'Count', 'Percent', 'Cum. Count', 'Cum. Percent')
+  column_names_df <- c('item', 'count', 'percent', 'cum_count', 'cum_percent')
+  column_align <- c(x_align, 'r', 'r', 'r', 'r')
 
-  if (any(class(x) == 'factor')) {
-    df <- tibble(item = x,
-                         fctlvl = x %>% as.integer()) %>%
-      group_by(item, fctlvl)
-    column_align <- c('l', 'r', 'r', 'r', 'r', 'r')
-  } else {
+  if (is.null(df)) {
+    # create table with counts and percentages
     df <- tibble(item = x) %>%
-      group_by(item)
-    # strip factor lvl from col names
-    column_names <- column_names[1:length(column_names) - 1]
-    column_names_df <- column_names_df[1:length(column_names_df) - 1]
-    column_align <- c(x_align, 'r', 'r', 'r', 'r')
+      group_by(item) %>%
+      summarise(count = n())
+
+    # sort according to setting
+    if (sort.count == TRUE) {
+      df <- df %>% arrange(desc(count), item)
+    } else {
+      df <- df %>% arrange(item)
+    }
+  } else {
+    column_names <- c("Group", column_names)
+    column_names_df <-c("group", column_names_df)
+    column_align <- c("l", column_align)
   }
-  df <- df %>% summarise(count = n())
 
   if (df$item %>% paste(collapse = ',') %like% '\033') {
     # remove escape char
@@ -433,36 +445,24 @@ frequency_tbl <- function(x,
     df <- df %>% mutate(item = item %>% gsub('\033', ' ', ., fixed = TRUE))
   }
 
-  # sort according to setting
-  if (sort.count == TRUE) {
-    df <- df %>% arrange(desc(count), item)
-  } else {
-    if (any(class(x) == 'factor')) {
-      df <- df %>% arrange(fctlvl, item)
-    } else {
-      df <- df %>% arrange(item)
-    }
-  }
-
   if (quote == TRUE) {
     df$item <- paste0('"', df$item, '"')
+    if (length(x.group) != 0) {
+      df$group <- paste0('"', df$group, '"')
+    }
   }
 
   df <- as.data.frame(df, stringsAsFactors = FALSE)
 
   df$percent <- df$count / base::sum(df$count, na.rm = TRUE)
-  df$cum_count <- base::cumsum(df$count)
+  if (length(x.group) == 0) {
+    df$cum_count <- base::cumsum(df$count)
+  }
   df$cum_percent <- df$cum_count / base::sum(df$count, na.rm = TRUE)
-
-  if (any(class(x) == 'factor')) {
-    # put factor last
-    df <- df %>% select(item, count, percent, cum_count, cum_percent, fctlvl)
+  if (length(x.group) != 0) {
+    # sort columns
+    df <- df[, column_names_df]
   }
-
-  colnames(df) <- column_names_df
-
-  class(df) <- c('frequency_tbl', class(df))
-  attr(df, 'package') <- 'AMR'
 
   if (markdown == TRUE) {
     tbl_format <- 'markdown'
@@ -470,17 +470,25 @@ frequency_tbl <- function(x,
     tbl_format <- 'pandoc'
   }
 
-  attr(df, 'opt') <- list(data = x.name,
-                          vars = cols,
-                          header = header,
-                          row_names = row.names,
-                          column_names = column_names,
-                          column_align = column_align,
-                          tbl_format = tbl_format,
-                          nmax = nmax,
-                          nmax.set = nmax.set)
+  if (!is.null(title)) {
+    title <- trimws(gsub("^Frequency table of", "", title[1L], ignore.case = TRUE))
+  }
 
-  df
+  structure(.Data = df,
+            class = c('frequency_tbl', class(df)),
+            opt = list(title = title,
+                       data = x.name,
+                       vars = cols,
+                       group_var = x.group,
+                       header = header,
+                       header_txt = header_txt,
+                       row_names = row.names,
+                       column_names = column_names,
+                       column_align = column_align,
+                       tbl_format = tbl_format,
+                       na = na,
+                       nmax = nmax,
+                       nmax.set = nmax.set))
 }
 
 #' @rdname freq
@@ -562,8 +570,10 @@ diff.frequency_tbl <- function(x, y, ...) {
 #' @exportMethod print.frequency_tbl
 #' @importFrom knitr kable
 #' @importFrom dplyr n_distinct
+#' @importFrom crayon bold silver
 #' @export
-print.frequency_tbl <- function(x, nmax = getOption("max.print.freq", default = 15), ...) {
+print.frequency_tbl <- function(x, nmax = getOption("max.print.freq", default = 15),
+                                markdown = !interactive(), header = !markdown, ...) {
 
   opt <- attr(x, 'opt')
 
@@ -571,14 +581,31 @@ print.frequency_tbl <- function(x, nmax = getOption("max.print.freq", default =
     opt$vars <- NULL
   }
 
-  if (!is.null(opt$data) & !is.null(opt$vars)) {
-    title <- paste0("of `", paste0(opt$vars, collapse = "` and `"), "` from ", opt$data)
-  } else if (!is.null(opt$data) & is.null(opt$vars)) {
-    title <- paste("of", opt$data)
-  } else if (is.null(opt$data) & !is.null(opt$vars)) {
-    title <- paste0("of `", paste0(opt$vars, collapse = "` and `"), "`")
+  if (is.null(opt$title)) {
+    if (!is.null(opt$data) & !is.null(opt$vars)) {
+      title <- paste0("`", paste0(opt$vars, collapse = "` and `"), "` from ", opt$data)
+    } else if (!is.null(opt$data) & is.null(opt$vars)) {
+      title <- opt$data
+    } else if (is.null(opt$data) & !is.null(opt$vars)) {
+      title <- paste0("`", paste0(opt$vars, collapse = "` and `"), "`")
+    } else {
+      title <- ""
+    }
+    if (title != "" & length(opt$group_var) != 0) {
+      group_var <- paste0("(grouped by `", opt$group_var, "`)")
+      if (opt$tbl_format == "pandoc") {
+        group_var <- silver(group_var)
+      }
+      title <- paste(title, group_var)
+    }
+    title <- trimws(title)
+    if (title == "") {
+      title <- "Frequency table"
+    } else {
+      title <- paste("Frequency table of", trimws(title))
+    }
   } else {
-    title <- ""
+    title <- opt$title
   }
 
   if (!missing(nmax)) {
@@ -593,11 +620,28 @@ print.frequency_tbl <- function(x, nmax = getOption("max.print.freq", default =
       opt$tbl_format <- "pandoc"
     }
   }
+  if (!missing(markdown)) {
+    opt$tbl_format <- "markdown"
+  }
+  if (!missing(header)) {
+    opt$header <- header
+  }
 
-  cat("Frequency table", title, "\n")
+  # bold title
+  if (opt$tbl_format == "pandoc") {
+    title <- bold(title)
+  } else if (opt$tbl_format == "markdown") {
+    title <- paste0("\n**", title, "**")
+  }
 
-  if (!is.null(opt$header)) {
-    cat(opt$header)
+  if (opt$header == TRUE) {
+    cat(title, "\n")
+    if (!is.null(opt$header_txt)) {
+      cat(opt$header_txt)
+    }
+  } else if (opt$tbl_format == "markdown") {
+    # do print title as caption in markdown
+    cat("\n", title, sep = "")
   }
 
   if (NROW(x) == 0) {
@@ -605,13 +649,12 @@ print.frequency_tbl <- function(x, nmax = getOption("max.print.freq", default =
     return(invisible())
   }
 
-  if (all(x$count == 1)) {
-    warning('All observations are unique.', call. = FALSE)
-  }
-
   # save old NA setting for kable
   opt.old <- options()$knitr.kable.NA
-  options(knitr.kable.NA = "<NA>")
+  if (is.null(opt$na)) {
+    opt$na <- "<NA>"
+  }
+  options(knitr.kable.NA = opt$na)
 
   if (nrow(x) > opt$nmax & opt$tbl_format != "markdown") {
 
@@ -640,17 +683,47 @@ print.frequency_tbl <- function(x, nmax = getOption("max.print.freq", default =
                     ' (',
                     (x.unprinted / (x.unprinted + x.printed)) %>% percent(force_zero = TRUE),
                     ') ]\n', sep = '')
+    if (opt$tbl_format == "pandoc") {
+      footer <- silver(footer) # only silver in regular printing
+    }
   } else {
     footer <- NULL
   }
 
-  if (any(class(x$item) %in% c('double', 'integer', 'numeric', 'raw', 'single'))) {
-    x$item <- format(x$item)
+  if ("item" %in% colnames(x)) {
+    if (any(class(x$item) %in% c('double', 'integer', 'numeric', 'raw', 'single'))) {
+      x$item <- format(x$item)
+    }
+  } else {
+    opt$column_names <- opt$column_names[!opt$column_names == "Item"]
+  }
+  if ("count" %in% colnames(x)) {
+    if (all(x$count == 1)) {
+      warning('All observations are unique.', call. = FALSE)
+    }
+    x$count <- format(x$count)
+  } else {
+    opt$column_names <- opt$column_names[!opt$column_names == "Count"]
+  }
+  if ("percent" %in% colnames(x)) {
+    x$percent <- percent(x$percent, force_zero = TRUE)
+  } else {
+    opt$column_names <- opt$column_names[!opt$column_names == "Percent"]
+  }
+  if ("cum_count" %in% colnames(x)) {
+    x$cum_count <- format(x$cum_count)
+  } else {
+    opt$column_names <- opt$column_names[!opt$column_names == "Cum. Count"]
+  }
+  if ("cum_percent" %in% colnames(x)) {
+    x$cum_percent <- percent(x$cum_percent, force_zero = TRUE)
+  } else {
+    opt$column_names <- opt$column_names[!opt$column_names == "Cum. Percent"]
+  }
+
+  if (opt$tbl_format == "markdown") {
+    cat("\n")
   }
-  x$count <- format(x$count)
-  x$percent <- percent(x$percent, force_zero = TRUE)
-  x$cum_count <- format(x$cum_count)
-  x$cum_percent <- percent(x$cum_percent, force_zero = TRUE)
 
   print(
     knitr::kable(x,
@@ -665,7 +738,11 @@ print.frequency_tbl <- function(x, nmax = getOption("max.print.freq", default =
     cat(footer)
   }
 
-  cat('\n')
+  if (opt$tbl_format == "markdown") {
+    cat("\n\n")
+  } else {
+    cat('\n')
+  }
 
   # reset old kable setting
   options(knitr.kable.NA = opt.old)
@@ -696,14 +773,27 @@ as_tibble.frequency_tbl <- function(x, validate = TRUE, ..., rownames = NA) {
 #' @exportMethod hist.frequency_tbl
 #' @export
 #' @importFrom graphics hist
-hist.frequency_tbl <- function(x, ...) {
+hist.frequency_tbl <- function(x, breaks = "Sturges", main = NULL, ...) {
   opt <- attr(x, 'opt')
+  if (!class(x$item) %in% c("numeric", "double", "integer", "Date")) {
+    stop("'x' must be numeric or Date.", call. = FALSE)
+  }
   if (!is.null(opt$vars)) {
     title <- opt$vars
+  } else if (!is.null(opt$data)) {
+    title <- opt$data
   } else {
-    title <- ""
+    title <- "frequency table"
   }
-  hist(as.vector(x), main = paste("Histogram of", title), xlab = title, ...)
+  if (class(x$item) == "Date") {
+    x <- as.Date(as.vector(x), origin = "1970-01-01")
+  } else {
+    x <- as.vector(x)
+  }
+  if (is.null(main)) {
+    main <- paste("Histogram of", title)
+  }
+  hist(x, main = main, xlab = title, ...)
 }
 
 #' @noRd
