@@ -38,6 +38,7 @@ globalVariables(c(".",
                   "fullname",
                   "genus",
                   "gramstain",
+                  "index",
                   "Interpretation",
                   "item",
                   "key_ab",
@@ -49,6 +50,12 @@ globalVariables(c(".",
                   "mic",
                   "microorganisms",
                   "microorganisms.old",
+                  "microorganismsDT",
+                  "microorganisms.prevDT",
+                  "microorganisms.unprevDT",
+                  "microorganisms.oldDT",
+                  "microorganisms.certe",
+                  "microorganisms.umcg",
                   "mo",
                   "mo.old",
                   "n",
@@ -58,6 +65,7 @@ globalVariables(c(".",
                   "Pasted",
                   "patient_id",
                   "prevalence",
+                  "psae",
                   "R",
                   "ref",
                   "real_first_isolate",
