@@ -22,23 +22,25 @@
 #' @param tbl table with antibiotic columns, like e.g. \code{amox} and \code{amcl}
 #' @param country country code to determine guidelines. EUCAST rules will be used when left empty, see Details. Should be or a code from the \href{https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2#Officially_assigned_code_elements}{list of ISO 3166-1 alpha-2 country codes}. Case-insensitive. Currently supported are \code{de} (Germany) and \code{nl} (the Netherlands).
 #' @param info print progress
-#' @inheritParams EUCAST_rules
+#' @inheritParams eucast_rules
 #' @param metr column name of an antibiotic. Use \code{NA} to skip a column, like \code{tica = NA}. Non-existing columns will anyway be skipped. See the Antibiotics section for an explanation of the abbreviations.
 #' @param ... parameters that are passed on to methods
-#' @inheritSection EUCAST_rules Antibiotics
+#' @inheritSection eucast_rules Antibiotics
 #' @details When \code{country} will be left blank, guidelines will be taken from EUCAST Expert Rules Version 3.1 "Intrinsic Resistance and Exceptional Phenotypes Tables" (\url{http://www.eucast.org/fileadmin/src/media/PDFs/EUCAST_files/Expert_Rules/Expert_rules_intrinsic_exceptional_V3.1.pdf}).
-#' @return Ordered factor with levels \code{Unknown < Negative < Unconfirmed < Positive}.
-#' @rdname MDRO
+#' @return Ordered factor with levels \code{Negative < Positive, unconfirmed < Positive}.
+#' @rdname mdro
+#' @importFrom dplyr %>%
+#' @importFrom crayon red blue
 #' @export
 #' @examples
 #' library(dplyr)
 #'
 #' septic_patients %>%
-#'   mutate(EUCAST = MDRO(.),
-#'          BRMO = BRMO(.))
-MDRO <- function(tbl,
+#'   mutate(EUCAST = mdro(.),
+#'          BRMO = brmo(.))
+mdro <- function(tbl,
                  country = NULL,
-                 col_mo = 'mo',
+                 col_mo = NULL,
                  info = TRUE,
                  amcl = 'amcl',
                  amik = 'amik',
@@ -85,6 +87,7 @@ MDRO <- function(tbl,
                  norf = 'norf',
                  oflo = 'oflo',
                  peni = 'peni',
+                 pipe = 'pipe',
                  pita = 'pita',
                  poly = 'poly',
                  qida = 'qida',
@@ -99,13 +102,21 @@ MDRO <- function(tbl,
                  trim = 'trim',
                  trsu = 'trsu',
                  vanc = 'vanc',
-                 col_bactid = 'bactid') {
+                 col_bactid = NULL) {
 
-  if (col_bactid %in% colnames(tbl)) {
+  if (!is.data.frame(tbl)) {
+    stop("`tbl` must be a data frame.", call. = FALSE)
+  }
+
+  # try to find columns based on type
+  # -- mo
+  if (!is.null(col_bactid)) {
     col_mo <- col_bactid
     warning("Use of `col_bactid` is deprecated. Use `col_mo` instead.")
-  }
-  if (!col_mo %in% colnames(tbl)) {
+  } else if (is.null(col_mo) & "mo" %in% lapply(tbl, class)) {
+    col_mo <- colnames(tbl)[lapply(tbl, class) == "mo"][1]
+    message("NOTE: Using column `", col_mo, "` as input for `col_mo`.")
+  } else if (!col_mo %in% colnames(tbl)) {
     stop('Column ', col_mo, ' not found.', call. = FALSE)
   }
 
@@ -118,8 +129,8 @@ MDRO <- function(tbl,
     country <- 'EUCAST'
   }
   country <- trimws(country)
-  if (country != 'EUCAST' & !country %like% '^[a-z]{2}$') {
-    stop('This is not a valid ISO 3166-1 alpha-2 country code: "', country, '". Please see ?MDRO.', call. = FALSE)
+  if (tolower(country) != 'eucast' & !country %like% '^[a-z]{2}$') {
+    stop('This is not a valid ISO 3166-1 alpha-2 country code: "', country, '". Please see ?mdro.', call. = FALSE)
   }
 
   # create list and make country code case-independent
@@ -141,24 +152,18 @@ MDRO <- function(tbl,
     guideline$name <- 'WIP-Richtlijn BRMO'
     guideline$version <- 'Revision of December 2017'
     guideline$source <- 'https://www.rivm.nl/Documenten_en_publicaties/Professioneel_Praktisch/Richtlijnen/Infectieziekten/WIP_Richtlijnen/WIP_Richtlijnen/Ziekenhuizen/WIP_richtlijn_BRMO_Bijzonder_Resistente_Micro_Organismen_ZKH'
-  # add here more countries like this:
-  # } else if (country$code == 'AA') {
-  #   country$name <- 'country name'
+    # add here more countries like this:
+    # } else if (country$code == 'xx') {
+    #   country$name <- 'country name'
   } else {
     stop('This country code is currently unsupported: ', guideline$country$code, call. = FALSE)
   }
 
-  # Console colours
-  # source: http://www.tldp.org/HOWTO/Bash-Prompt-HOWTO/x329.html
-  ANSI_red <- "\033[31m"
-  ANSI_blue <- "\033[34m"
-  ANSI_reset <- "\033[0m"
-
   if (info == TRUE) {
     cat("Determining multidrug-resistant organisms (MDRO), according to:\n",
-        "Guideline: ", ANSI_red, guideline$name, ", ", guideline$version, ANSI_reset, "\n",
-        "Country  : ", ANSI_red, guideline$country$name, ANSI_reset, "\n",
-        "Source   : ", ANSI_blue, guideline$source, ANSI_reset, "\n",
+        "Guideline: ", red(paste0(guideline$name, ", ", guideline$version, "\n")),
+        "Country  : ", red(paste0(guideline$country$name, "\n")),
+        "Source   : ", blue(paste0(guideline$source, "\n")),
         "\n", sep = "")
   }
 
@@ -231,83 +236,102 @@ MDRO <- function(tbl,
   vanc <- col.list[vanc]
 
   # antibiotic classes
-  aminoglycosides <- c(tobr, gent, kana, neom, neti, siso)
-  tetracyclines <- c(doxy, mino, tetr) # since EUCAST v3.1 tige(cycline) is set apart
-  polymyxins <- c(poly, coli)
-  macrolides <- c(eryt, azit, roxi, clar) # since EUCAST v3.1 clin(damycin) is set apart
-  glycopeptides <- c(vanc, teic)
-  streptogramins <- qida # should officially also be pristinamycin and quinupristin/dalfopristin
+  aminoglycosides <- c(tobr, gent) # can also be kana but that one is often intrinsic R
   cephalosporins <- c(cfep, cfot, cfox, cfra, cfta, cftr, cfur, czol)
   cephalosporins_3rd <- c(cfot, cftr, cfta)
   carbapenems <- c(erta, imip, mero)
-  aminopenicillins <- c(ampi, amox)
-  ureidopenicillins <- pita # should officially also be azlo and mezlo
-  fluoroquinolones <- c(oflo, cipr, norf, levo, moxi)
+  fluoroquinolones <- c(oflo, cipr, levo, moxi)
 
   # helper function for editing the table
-  trans_tbl <- function(to, rows, cols) {
+  trans_tbl <- function(to, rows, cols, any_all) {
     cols <- cols[!is.na(cols)]
     if (length(rows) > 0 & length(cols) > 0) {
-      col_filter <- which(tbl[, cols] == 'R')
+      if (any_all == "any") {
+        col_filter <- which(tbl[, cols] == 'R')
+      } else if (any_all == "all") {
+        col_filter <- tbl %>%
+          mutate(index = 1:nrow(.)) %>%
+          filter_at(vars(cols), all_vars(. == "R")) %>%
+          pull((index))
+      }
       rows <- rows[rows %in% col_filter]
       tbl[rows, 'MDRO'] <<- to
     }
   }
 
-  # join microorganisms
-  tbl <- tbl %>% left_join_microorganisms(col_mo)
+  if (!tbl %>% pull(col_mo) %>% is.mo()) {
+    tbl[, col_mo] <- as.mo(tbl[, col_mo])
+  }
 
-  tbl$MDRO <- NA_integer_
+  tbl <- tbl %>%
+    # join to microorganisms data set
+    left_join_microorganisms(by = col_mo) %>%
+    # add unconfirmed to where genus is available
+    mutate(MDRO = ifelse(!is.na(genus), 1, NA_integer_))
 
   if (guideline$country$code == 'eucast') {
     # EUCAST ------------------------------------------------------------------
     # Table 5
-    trans_tbl(4,
+    trans_tbl(3,
               which(tbl$family == 'Enterobacteriaceae'
                     | tbl$fullname %like% '^Pseudomonas aeruginosa'
                     | tbl$genus == 'Acinetobacter'),
-              coli)
-    trans_tbl(4,
+              coli,
+              "all")
+    trans_tbl(3,
               which(tbl$fullname %like% '^Salmonella Typhi'),
-              c(carbapenems, fluoroquinolones))
-    trans_tbl(4,
+              c(carbapenems, fluoroquinolones),
+              "any")
+    trans_tbl(3,
               which(tbl$fullname %like% '^Haemophilus influenzae'),
-              c(cephalosporins_3rd, carbapenems, fluoroquinolones))
-    trans_tbl(4,
+              c(cephalosporins_3rd, carbapenems, fluoroquinolones),
+              "any")
+    trans_tbl(3,
               which(tbl$fullname %like% '^Moraxella catarrhalis'),
-              c(cephalosporins_3rd, fluoroquinolones))
-    trans_tbl(4,
+              c(cephalosporins_3rd, fluoroquinolones),
+              "any")
+    trans_tbl(3,
               which(tbl$fullname %like% '^Neisseria meningitidis'),
-              c(cephalosporins_3rd, fluoroquinolones))
-    trans_tbl(4,
+              c(cephalosporins_3rd, fluoroquinolones),
+              "any")
+    trans_tbl(3,
               which(tbl$fullname %like% '^Neisseria gonorrhoeae'),
-              azit)
+              azit,
+              "any")
     # Table 6
-    trans_tbl(4,
+    trans_tbl(3,
               which(tbl$fullname %like% '^Staphylococcus (aureus|epidermidis|coagulase negatief|hominis|haemolyticus|intermedius|pseudointermedius)'),
-              c(vanc, teic, dapt, line, qida, tige))
-    trans_tbl(4,
+              c(vanc, teic, dapt, line, qida, tige),
+              "any")
+    trans_tbl(3,
               which(tbl$genus == 'Corynebacterium'),
-              c(vanc, teic, dapt, line, qida, tige))
-    trans_tbl(4,
+              c(vanc, teic, dapt, line, qida, tige),
+              "any")
+    trans_tbl(3,
               which(tbl$fullname %like% '^Streptococcus pneumoniae'),
-              c(carbapenems, vanc, teic, dapt, line, qida, tige, rifa))
-    trans_tbl(4, # Sr. groups A/B/C/G
+              c(carbapenems, vanc, teic, dapt, line, qida, tige, rifa),
+              "any")
+    trans_tbl(3, # Sr. groups A/B/C/G
               which(tbl$fullname %like% '^Streptococcus (pyogenes|agalactiae|equisimilis|equi|zooepidemicus|dysgalactiae|anginosus)'),
-              c(peni, cephalosporins, vanc, teic, dapt, line, qida, tige))
-    trans_tbl(4,
+              c(peni, cephalosporins, vanc, teic, dapt, line, qida, tige),
+              "any")
+    trans_tbl(3,
               which(tbl$genus == 'Enterococcus'),
-              c(dapt, line, tige, teic))
-    trans_tbl(4,
+              c(dapt, line, tige, teic),
+              "any")
+    trans_tbl(3,
               which(tbl$fullname %like% '^Enterococcus faecalis'),
-              c(ampi, amox))
+              c(ampi, amox),
+              "any")
     # Table 7
-    trans_tbl(4,
+    trans_tbl(3,
               which(tbl$genus == 'Bacteroides'),
-              metr)
-    trans_tbl(4,
+              metr,
+              "any")
+    trans_tbl(3,
               which(tbl$fullname %like% '^Clostridium difficile'),
-              c(metr, vanc))
+              c(metr, vanc),
+              "any")
   }
 
   if (guideline$country$code == 'de') {
@@ -322,107 +346,88 @@ MDRO <- function(tbl,
     carbapenems <- carbapenems[!is.na(carbapenems)]
 
     # Table 1
-    tbl[which(
-      tbl$family == 'Enterobacteriaceae'
-      & rowSums(tbl[, aminoglycosides] == 'R', na.rm = TRUE) >= 1
-      & rowSums(tbl[, fluoroquinolones] == 'R', na.rm = TRUE) >= 1
-    ), 'MDRO'] <- 4
-    tbl[which(
-      tbl$family == 'Enterobacteriaceae'
-      & rowSums(tbl[, carbapenems] == 'R', na.rm = TRUE) >= 1
-    ), 'MDRO'] <- 3
-    # rest is negative
-    tbl[which(
-      tbl$family == 'Enterobacteriaceae'
-      & tbl$MDRO == 1
-    ), 'MDRO'] <- 2
+    trans_tbl(3,
+              which(tbl$family == 'Enterobacteriaceae'),
+              c(aminoglycosides, fluoroquinolones),
+              "all")
+
+    trans_tbl(2,
+              which(tbl$family == 'Enterobacteriaceae'),
+              c(carbapenems),
+              "any")
 
     # Table 2
+    trans_tbl(2,
+              which(tbl$genus == 'Acinetobacter'),
+              c(carbapenems),
+              "any")
+    trans_tbl(3,
+              which(tbl$genus == 'Acinetobacter'),
+              c(aminoglycosides, fluoroquinolones),
+              "all")
+
+    trans_tbl(3,
+              which(tbl$fullname %like% '^Stenotrophomonas maltophilia'),
+              trsu,
+              "all")
+
+    if (!is.na(mero) & !is.na(imip)
+        & !is.na(gent) & !is.na(tobr)
+        & !is.na(cipr)
+        & !is.na(cfta)
+        & !is.na(pita) ) {
+      tbl <- tbl %>% mutate(
+        psae = 0,
+        psae = ifelse(mero == "R" | imip == "R", psae + 1, psae),
+        psae = ifelse(gent == "R" & tobr == "R", psae + 1, psae),
+        psae = ifelse(cipr == "R", psae + 1, psae),
+        psae = ifelse(cfta == "R", psae + 1, psae),
+        psae = ifelse(pita == "R", psae + 1, psae),
+        psae = ifelse(is.na(psae), 0, psae)
+      )
+    } else {
+      tbl$psae <- 0
+    }
     tbl[which(
-      tbl$genus == 'Acinetobacter'
-      & rowSums(tbl[, carbapenems] == 'R', na.rm = TRUE) >= 1
+      tbl$fullname %like% 'Pseudomonas aeruginosa'
+      & tbl$psae >= 3
     ), 'MDRO'] <- 3
-    tbl[which(
-      tbl$genus == 'Acinetobacter'
-      & rowSums(tbl[, aminoglycosides] == 'R', na.rm = TRUE) >= 1
-      & rowSums(tbl[, fluoroquinolones] == 'R', na.rm = TRUE) >= 1
-    ), 'MDRO'] <- 4
-    # rest of Acinetobacter is negative
-    tbl[which(
-      tbl$genus == 'Acinetobacter'
-      & tbl$MDRO == 1
-    ), 'MDRO'] <- 2
-
-    tbl[which(
-      tbl$fullname %like% 'Stenotrophomonas maltophilia'
-      & tbl[, trsu] == 'R'
-    ), 'MDRO'] <- 4
-    # rest of Stenotrophomonas is negative
-    tbl[which(
-      tbl$fullname %like% 'Stenotrophomonas maltophilia'
-      & tbl$MDRO == 1
-    ), 'MDRO'] <- 2
-
-    tbl[which(
-      tbl$fullname %like% 'Pseudomonas aeruginosa'
-      & sum(rowSums(tbl[, carbapenems] == 'R', na.rm = TRUE) >= 1,
-           rowSums(tbl[, aminoglycosides] == 'R', na.rm = TRUE) >= 1,
-           rowSums(tbl[, fluoroquinolones] == 'R', na.rm = TRUE) >= 1,
-           tbl[, cfta] == 'R',
-           tbl[, pita] == 'R') >= 3
-    ), 'MDRO'] <- 4
-    # rest of Pseudomonas is negative
-    tbl[which(
-      tbl$fullname %like% 'Pseudomonas aeruginosa'
-      & tbl$MDRO == 1
-    ), 'MDRO'] <- 2
 
     # Table 3
-    tbl[which(
-      tbl$fullname %like% 'Streptococcus pneumoniae'
-      & tbl[, peni] == 'R'
-    ), 'MDRO'] <- 4
-    tbl[which(
-      tbl$fullname %like% 'Streptococcus pneumoniae'
-      & tbl[, vanc] == 'R'
-    ), 'MDRO'] <- 4
-    # rest of Streptococcus pneumoniae is negative
-    tbl[which(
-      tbl$fullname %like% 'Streptococcus pneumoniae'
-      & tbl$MDRO == 1
-    ), 'MDRO'] <- 2
-
-    tbl[which(
-      tbl$fullname %like% 'Enterococcus faecium'
-      &  rowSums(tbl[, c(peni, vanc)] == 'R', na.rm = TRUE) >= 1
-    ), 'MDRO'] <- 4
-    # rest of Enterococcus faecium is negative
-    tbl[which(
-      tbl$fullname %like% 'Enterococcus faecium'
-      & tbl$MDRO == 1
-    ), 'MDRO'] <- 2
+    trans_tbl(3,
+              which(tbl$fullname %like% 'Streptococcus pneumoniae'),
+              peni,
+              "all")
+    trans_tbl(3,
+              which(tbl$fullname %like% 'Streptococcus pneumoniae'),
+              vanc,
+              "all")
+    trans_tbl(3,
+              which(tbl$fullname %like% 'Enterococcus faecium'),
+              c(peni, vanc),
+              "all")
   }
 
   factor(x =  tbl$MDRO,
-         levels = c(1:4),
-         labels = c('Unknown', 'Negative', 'Unconfirmed', 'Positive'),
+         levels = 1:3,
+         labels = c('Negative', 'Positive, unconfirmed', 'Positive'),
          ordered = TRUE)
 }
 
-#' @rdname MDRO
+#' @rdname mdro
 #' @export
-BRMO <- function(tbl, country = "nl", ...) {
-  MDRO(tbl = tbl, country = "nl", ...)
+brmo <- function(..., country = "nl") {
+  mdro(..., country = "nl")
 }
 
-#' @rdname MDRO
+#' @rdname mdro
 #' @export
-MRGN <- function(tbl, country = "de", ...) {
-  MDRO(tbl = tbl, country = "de", ...)
+mrgn <- function(tbl, country = "de", ...) {
+  mdro(tbl = tbl, country = "de", ...)
 }
 
-#' @rdname MDRO
+#' @rdname mdro
 #' @export
-EUCAST_exceptional_phenotypes <- function(tbl, country = "EUCAST", ...) {
-  MDRO(tbl = tbl, country = "EUCAST", ...)
+eucast_exceptional_phenotypes <- function(tbl, country = "EUCAST", ...) {
+  mdro(tbl = tbl, country = "EUCAST", ...)
 }
