@@ -22,10 +22,12 @@
 #'
 #' \code{portion_R} and \code{portion_IR} can be used to calculate resistance, \code{portion_S} and \code{portion_SI} can be used to calculate susceptibility.\cr
 #' @param ... one or more vectors (or columns) with antibiotic interpretations. They will be transformed internally with \code{\link{as.rsi}} if needed. Use multiple columns to calculate (the lack of) co-resistance: the probability where one of two drugs have a resistant or susceptible result. See Examples.
-#' @param minimum minimal amount of available isolates. Any number lower than \code{minimum} will return \code{NA}. The default number of \code{30} isolates is advised by the CLSI as best practice, see Source.
-#' @param as_percent logical to indicate whether the output must be returned as a hundred fold with \% sign (a character). A value of \code{0.123456} will then be returned as \code{"12.3\%"}.
+#' @param minimum the minimal amount of available isolates. Any number lower than \code{minimum} will return \code{NA} with a warning. The default number of \code{30} isolates is advised by the Clinical and Laboratory Standards Institute (CLSI) as best practice, see Source.
+#' @param as_percent a logical to indicate whether the output must be returned as a hundred fold with \% sign (a character). A value of \code{0.123456} will then be returned as \code{"12.3\%"}.
+#' @param also_single_tested a logical to indicate whether (in combination therapies) also observations should be included where not all antibiotics were tested, but at least one of the tested antibiotics contains a target interpretation (e.g. S in case of \code{portion_S} and R in case of \code{portion_R}). \strong{This would lead to selection bias in almost all cases.}
 #' @param data a \code{data.frame} containing columns with class \code{rsi} (see \code{\link{as.rsi}})
 #' @param translate_ab a column name of the \code{\link{antibiotics}} data set to translate the antibiotic abbreviations to, using \code{\link{abname}}. This can be set with \code{\link{getOption}("get_antibiotic_names")}.
+#' @param combine_IR a logical to indicate whether all values of I and R must be merged into one, so the output only consists of S vs. IR (susceptible vs. non-susceptible)
 #' @details \strong{Remember that you should filter your table to let it contain only first isolates!} Use \code{\link{first_isolate}} to determine them in your data set.
 #'
 #' These functions are not meant to count isolates, but to calculate the portion of resistance/susceptibility. If a column has been transformed with \code{\link{as.rsi}}, just use e.g. \code{isolates[isolates == "R"]} to get the resistant ones. You could then calculate the \code{\link{length}} of it.
@@ -50,8 +52,7 @@
 #' @source \strong{M39 Analysis and Presentation of Cumulative Antimicrobial Susceptibility Test Data, 4th Edition}, 2014, \emph{Clinical and Laboratory Standards Institute (CLSI)}. \url{https://clsi.org/standards/products/microbiology/documents/m39/}.
 #'
 #' Wickham H. \strong{Tidy Data.} The Journal of Statistical Software, vol. 59, 2014. \url{http://vita.had.co.nz/papers/tidy-data.html}
-#' @seealso \code{\link[AMR]{count}_*} to count resistant and susceptibile isolates.\cr
-#' \code{\link{n_rsi}} to count all cases where antimicrobial results are available.
+#' @seealso \code{\link[AMR]{count}_*} to count resistant and susceptibile isolates.
 #' @keywords resistance susceptibility rsi_df rsi antibiotics isolate isolates
 #' @return Double or, when \code{as_percent = TRUE}, a character.
 #' @rdname portion
@@ -92,24 +93,24 @@
 #'
 #' # Calculate co-resistance between amoxicillin/clav acid and gentamicin,
 #' # so we can see that combination therapy does a lot more than mono therapy:
-#' septic_patients %>% portion_S(amcl)       # S = 67.3%
-#' septic_patients %>% n_rsi(amcl)           # n = 1570
+#' septic_patients %>% portion_S(amcl)       # S = 67.1%
+#' septic_patients %>% count_all(amcl)       # n = 1576
 #'
 #' septic_patients %>% portion_S(gent)       # S = 74.0%
-#' septic_patients %>% n_rsi(gent)           # n = 1842
+#' septic_patients %>% count_all(gent)       # n = 1855
 #'
-#' septic_patients %>% portion_S(amcl, gent) # S = 92.1%
-#' septic_patients %>% n_rsi(amcl, gent)     # n = 1504
+#' septic_patients %>% portion_S(amcl, gent) # S = 92.0%
+#' septic_patients %>% count_all(amcl, gent) # n = 1517
 #'
 #'
 #' septic_patients %>%
 #'   group_by(hospital_id) %>%
 #'   summarise(cipro_p = portion_S(cipr, as_percent = TRUE),
-#'             cipro_n = n_rsi(cipr),
+#'             cipro_n = count_all(cipr),
 #'             genta_p = portion_S(gent, as_percent = TRUE),
-#'             genta_n = n_rsi(gent),
+#'             genta_n = count_all(gent),
 #'             combination_p = portion_S(cipr, gent, as_percent = TRUE),
-#'             combination_n = n_rsi(cipr, gent))
+#'             combination_n = count_all(cipr, gent))
 #'
 #' # Get portions S/I/R immediately of all rsi columns
 #' septic_patients %>%
@@ -130,16 +131,18 @@
 #'   filter(first_isolate == TRUE,
 #'          genus == "Helicobacter") %>%
 #'   summarise(p = portion_S(amox, metr),  # amoxicillin with metronidazole
-#'             n = n_rsi(amox, metr))
+#'             n = count_all(amox, metr))
 #' }
 portion_R <- function(...,
                       minimum = 30,
-                      as_percent = FALSE) {
+                      as_percent = FALSE,
+                      also_single_tested = FALSE) {
   rsi_calc(...,
            type = "R",
            include_I = FALSE,
            minimum = minimum,
            as_percent = as_percent,
+           also_single_tested = also_single_tested,
            only_count = FALSE)
 }
 
@@ -147,12 +150,14 @@ portion_R <- function(...,
 #' @export
 portion_IR <- function(...,
                        minimum = 30,
-                       as_percent = FALSE) {
+                       as_percent = FALSE,
+                       also_single_tested = FALSE) {
   rsi_calc(...,
            type = "R",
            include_I = TRUE,
            minimum = minimum,
            as_percent = as_percent,
+           also_single_tested = also_single_tested,
            only_count = FALSE)
 }
 
@@ -160,12 +165,14 @@ portion_IR <- function(...,
 #' @export
 portion_I <- function(...,
                       minimum = 30,
-                      as_percent = FALSE) {
+                      as_percent = FALSE,
+                      also_single_tested = FALSE) {
   rsi_calc(...,
            type = "I",
            include_I = FALSE,
            minimum = minimum,
            as_percent = as_percent,
+           also_single_tested = also_single_tested,
            only_count = FALSE)
 }
 
@@ -173,12 +180,14 @@ portion_I <- function(...,
 #' @export
 portion_SI <- function(...,
                        minimum = 30,
-                       as_percent = FALSE) {
+                       as_percent = FALSE,
+                       also_single_tested = FALSE) {
   rsi_calc(...,
            type = "S",
            include_I = TRUE,
            minimum = minimum,
            as_percent = as_percent,
+           also_single_tested = also_single_tested,
            only_count = FALSE)
 }
 
@@ -186,12 +195,14 @@ portion_SI <- function(...,
 #' @export
 portion_S <- function(...,
                       minimum = 30,
-                      as_percent = FALSE) {
+                      as_percent = FALSE,
+                      also_single_tested = FALSE) {
   rsi_calc(...,
            type = "S",
            include_I = FALSE,
            minimum = minimum,
            as_percent = as_percent,
+           also_single_tested = also_single_tested,
            only_count = FALSE)
 }
 
@@ -201,7 +212,8 @@ portion_S <- function(...,
 portion_df <- function(data,
                        translate_ab = getOption("get_antibiotic_names", "official"),
                        minimum = 30,
-                       as_percent = FALSE) {
+                       as_percent = FALSE,
+                       combine_IR = FALSE) {
 
   if (!"data.frame" %in% class(data)) {
     stop("`portion_df` must be called on a data.frame")
@@ -224,27 +236,43 @@ portion_df <- function(data,
     mutate(Interpretation = "S") %>%
     select(Interpretation, everything())
 
-  resI <- summarise_if(.tbl = data,
-                       .predicate = is.rsi,
-                       .funs = portion_I,
-                       minimum = minimum,
-                       as_percent = as_percent) %>%
-    mutate(Interpretation = "I") %>%
-    select(Interpretation, everything())
+  if (combine_IR == FALSE) {
+    resI <- summarise_if(.tbl = data,
+                         .predicate = is.rsi,
+                         .funs = portion_I,
+                         minimum = minimum,
+                         as_percent = as_percent) %>%
+      mutate(Interpretation = "I") %>%
+      select(Interpretation, everything())
 
-  resR <- summarise_if(.tbl = data,
-                       .predicate = is.rsi,
-                       .funs = portion_R,
-                       minimum = minimum,
-                       as_percent = as_percent) %>%
-    mutate(Interpretation = "R") %>%
-    select(Interpretation, everything())
+    resR <- summarise_if(.tbl = data,
+                         .predicate = is.rsi,
+                         .funs = portion_R,
+                         minimum = minimum,
+                         as_percent = as_percent) %>%
+      mutate(Interpretation = "R") %>%
+      select(Interpretation, everything())
 
-  data.groups <- group_vars(data)
+    data.groups <- group_vars(data)
 
-  res <- bind_rows(resS, resI, resR) %>%
-    mutate(Interpretation = factor(Interpretation, levels = c("R", "I", "S"), ordered = TRUE)) %>%
-    tidyr::gather(Antibiotic, Value, -Interpretation, -data.groups)
+    res <- bind_rows(resS, resI, resR) %>%
+      mutate(Interpretation = factor(Interpretation, levels = c("R", "I", "S"), ordered = TRUE)) %>%
+      tidyr::gather(Antibiotic, Value, -Interpretation, -data.groups)
+  } else {
+    resIR <- summarise_if(.tbl = data,
+                          .predicate = is.rsi,
+                          .funs = portion_IR,
+                          minimum = minimum,
+                          as_percent = as_percent) %>%
+      mutate(Interpretation = "IR") %>%
+      select(Interpretation, everything())
+
+    data.groups <- group_vars(data)
+
+    res <- bind_rows(resS, resIR) %>%
+      mutate(Interpretation = factor(Interpretation, levels = c("IR", "S"), ordered = TRUE)) %>%
+      tidyr::gather(Antibiotic, Value, -Interpretation, -data.groups)
+  }
 
   if (!translate_ab == FALSE) {
     if (!tolower(translate_ab) %in% tolower(colnames(AMR::antibiotics))) {
@@ -273,6 +301,8 @@ rsi <- function(ab1,
                 as_percent = FALSE,
                 ...) {
 
+  .Deprecated(new = paste0("portion_", interpretation))
+
   if (all(is.null(ab2))) {
     df <- tibble(ab1 = ab1)
   } else {
@@ -280,19 +310,16 @@ rsi <- function(ab1,
                  ab2 = ab2)
   }
 
+  if (!interpretation %in% c("S", "SI", "IS", "I", "RI", "IR", "R")) {
+    stop("invalid interpretation")
+  }
+
   result <- case_when(
     interpretation == "S"             ~ portion_S(df, minimum = minimum, as_percent = FALSE),
     interpretation %in% c("SI", "IS") ~ portion_SI(df, minimum = minimum, as_percent = FALSE),
     interpretation == "I"             ~ portion_I(df, minimum = minimum, as_percent = FALSE),
     interpretation %in% c("RI", "IR") ~ portion_IR(df, minimum = minimum, as_percent = FALSE),
-    interpretation == "R"             ~ portion_R(df, minimum = minimum, as_percent = FALSE),
-    TRUE ~ -1
-  )
-  if (result == -1) {
-    stop("invalid interpretation")
-  }
-
-  .Deprecated(new = paste0("portion_", interpretation))
+    interpretation == "R"             ~ portion_R(df, minimum = minimum, as_percent = FALSE))
 
   if (as_percent == TRUE) {
     percent(result, force_zero = TRUE)
