@@ -22,7 +22,7 @@
 #' @rdname as.rsi
 #' @param x vector
 #' @details The function \code{is.rsi.eligible} returns \code{TRUE} when a columns contains only valid antimicrobial interpretations (S and/or I and/or R), and \code{FALSE} otherwise.
-#' @return Ordered factor with new class \code{rsi} and new attribute \code{package}
+#' @return Ordered factor with new class \code{rsi}
 #' @keywords rsi
 #' @export
 #' @importFrom dplyr %>%
@@ -81,9 +81,8 @@ as.rsi <- function(x) {
               list_missing, call. = FALSE)
     }
 
-    x <- x %>% factor(levels = c("S", "I", "R"), ordered = TRUE)
+    x <- factor(x, levels = c("S", "I", "R"), ordered = TRUE)
     class(x) <- c('rsi', 'ordered', 'factor')
-    attr(x, 'package') <- 'AMR'
     x
   }
 }
@@ -99,16 +98,19 @@ is.rsi <- function(x) {
 #' @export
 #' @importFrom dplyr %>%
 is.rsi.eligible <- function(x) {
-  # remove all but a-z
-  distinct_val <- x %>% unique() %>% sort() %>% as.character() %>% gsub("(\\W|\\d)+", "", .)
-  # remove NAs and empty values
-  distinct_val <- distinct_val[!is.na(distinct_val) & trimws(distinct_val) != ""]
-  # get RSI class
-  distinct_val_rsi <- as.character(suppressWarnings(as.rsi(distinct_val)))
-
-  # is not empty and identical to new class
-  length(distinct_val) > 0 &
-    identical(distinct_val, distinct_val_rsi)
+  if (is.logical(x)
+      | is.numeric(x)
+      | is.mo(x)
+      | identical(class(x), "Date")
+      | identical(levels(x), c("S", "I", "R"))) {
+    # no transformation needed
+    FALSE
+  } else {
+    # check all but a-z
+    x <- unique(gsub("[^RSIrsi]+", "", unique(x)))
+    all(x %in% c("R", "I", "S", "", NA_character_)) &
+      !all(x %in% c("", NA_character_))
+  }
 }
 
 #' @exportMethod print.rsi
