@@ -2,16 +2,21 @@ context("count.R")
 
 test_that("counts work", {
   # amox resistance in `septic_patients`
-  expect_equal(count_R(septic_patients$amox), 662)
+  expect_equal(count_R(septic_patients$amox), 683)
   expect_equal(count_I(septic_patients$amox), 3)
-  expect_equal(count_S(septic_patients$amox), 335)
+  expect_equal(count_S(septic_patients$amox), 486)
   expect_equal(count_R(septic_patients$amox) + count_I(septic_patients$amox),
                count_IR(septic_patients$amox))
   expect_equal(count_S(septic_patients$amox) + count_I(septic_patients$amox),
                count_SI(septic_patients$amox))
 
-  expect_equal(septic_patients %>% count_S(amcl), 1057)
-  expect_equal(septic_patients %>% count_S(amcl, gent), 1396)
+  library(dplyr)
+  expect_equal(septic_patients %>% count_S(amcl), 1291)
+  expect_equal(septic_patients %>% count_S(amcl, gent), 1609)
+  expect_equal(septic_patients %>% count_all(amcl, gent), 1747)
+  expect_identical(septic_patients %>% count_all(amcl, gent),
+                   septic_patients %>% count_S(amcl, gent) +
+                     septic_patients %>% count_IR(amcl, gent))
 
   # count of cases
   expect_equal(septic_patients %>%
@@ -22,8 +27,18 @@ test_that("counts work", {
                  pull(combination),
                c(192, 446, 184, 474))
 
-  expect_equal(septic_patients %>% select(amox, cipr) %>% count_df(translate_ab = "official") %>% nrow(),
-               6)
+  # count_df
+  expect_equal(
+    septic_patients %>% select(amox) %>% count_df() %>% pull(Value),
+    c(septic_patients$amox %>% count_S(),
+      septic_patients$amox %>% count_I(),
+      septic_patients$amox %>% count_R())
+  )
+  expect_equal(
+    septic_patients %>% select(amox) %>% count_df(combine_IR = TRUE) %>% pull(Value),
+    c(septic_patients$amox %>% count_S(),
+      septic_patients$amox %>% count_IR())
+  )
 
   # warning for speed loss
   expect_warning(count_R(as.character(septic_patients$amcl)))
