@@ -1,6 +1,7 @@
 context("mo_property.R")
 
 test_that("mo_property works", {
+  expect_equal(mo_kingdom("E. coli"), "Bacteria")
   expect_equal(mo_subkingdom("E. coli"), "Negibacteria")
   expect_equal(mo_phylum("E. coli"), "Proteobacteria")
   expect_equal(mo_class("E. coli"), "Gammaproteobacteria")
@@ -12,6 +13,7 @@ test_that("mo_property works", {
   expect_equal(mo_fullname("E. coli"), "Escherichia coli")
   expect_equal(mo_type("E. coli", language = "en"), "Bacteria")
   expect_equal(mo_gramstain("E. coli", language = "en"), "Gram negative")
+  expect_equal(mo_TSN("E. coli"), 285)
   expect_equal(class(mo_taxonomy("E. coli")), "list")
   expect_equal(names(mo_taxonomy("E. coli")), c("subkingdom", "phylum", "class", "order",
                                                 "family", "genus", "species", "subspecies"))
@@ -20,7 +22,7 @@ test_that("mo_property works", {
 
   expect_equal(mo_shortname("MRSA"), "S. aureus")
   expect_equal(mo_shortname("MRSA", Becker = TRUE), "S. aureus")
-  expect_equal(mo_shortname("MRSA", Becker = "all"), "CoPS")
+  expect_equal(mo_shortname("MRSA", Becker = "all", language = "en"), "CoPS")
   expect_equal(mo_shortname("S. aga"), "S. agalactiae")
   expect_equal(mo_shortname("S. aga", Lancefield = TRUE), "GBS")
 
@@ -42,4 +44,20 @@ test_that("mo_property works", {
 
   expect_error(mo_gramstain("E. coli", language = "UNKNOWN"))
 
+  # manual property function
+  expect_error(mo_property("E. coli", property = c("tsn", "fullname")))
+  expect_error(mo_property("E. coli", property = "UNKNOWN"))
+  expect_identical(mo_property("E. coli", property = "fullname"),
+                   mo_fullname("E. coli"))
+  expect_identical(mo_property("E. coli", property = "genus"),
+                   mo_genus("E. coli"))
+  expect_identical(mo_property("E. coli", property = "species"),
+                   mo_species("E. coli"))
+
+  # check vector with random values
+  library(dplyr)
+  df_sample <- AMR::microorganisms %>% sample_n(100)
+  expect_identical(df_sample %>% pull(mo) %>% mo_fullname(language = "en"),
+                   df_sample %>% pull(fullname))
+
 })
