@@ -2,8 +2,8 @@ context("portion.R")
 
 test_that("portions works", {
   # amox resistance in `septic_patients`
-  expect_equal(portion_R(septic_patients$amox), 0.662, tolerance = 0.0001)
-  expect_equal(portion_I(septic_patients$amox), 0.003, tolerance = 0.0001)
+  expect_equal(portion_R(septic_patients$amox), 0.5827645, tolerance = 0.0001)
+  expect_equal(portion_I(septic_patients$amox), 0.0025597, tolerance = 0.0001)
   expect_equal(1 - portion_R(septic_patients$amox) - portion_I(septic_patients$amox),
                portion_S(septic_patients$amox))
   expect_equal(portion_R(septic_patients$amox) + portion_I(septic_patients$amox),
@@ -12,17 +12,20 @@ test_that("portions works", {
                portion_SI(septic_patients$amox))
 
   expect_equal(septic_patients %>% portion_S(amcl),
-               0.6706853,
+               0.7062363,
                tolerance = 0.001)
   expect_equal(septic_patients %>% portion_S(amcl, gent),
-               0.9202373,
+               0.9210074,
+               tolerance = 0.001)
+  expect_equal(septic_patients %>% portion_S(amcl, gent, also_single_tested = TRUE),
+               0.9239669,
                tolerance = 0.001)
 
   # amcl+genta susceptibility around 92.1%
   expect_equal(suppressWarnings(rsi(septic_patients$amcl,
                                     septic_patients$gent,
                                     interpretation = "S")),
-               0.9202373,
+               0.9210074,
                tolerance = 0.000001)
 
   # percentages
@@ -57,7 +60,7 @@ test_that("portions works", {
                                     septic_patients$gent)))
   expect_equal(suppressWarnings(n_rsi(as.character(septic_patients$amcl,
                                                    septic_patients$gent))),
-               1576)
+               1828)
 
   # check for errors
   expect_error(portion_IR("test", minimum = "test"))
@@ -66,13 +69,14 @@ test_that("portions works", {
   expect_error(portion_I("test", as_percent = "test"))
   expect_error(portion_S("test", minimum = "test"))
   expect_error(portion_S("test", as_percent = "test"))
+  expect_error(portion_S("test", also_single_tested = "test"))
 
   # check too low amount of isolates
-  expect_identical(portion_R(septic_patients$amox, minimum = nrow(septic_patients) + 1),
+  expect_identical(suppressWarnings(portion_R(septic_patients$amox, minimum = nrow(septic_patients) + 1)),
                    NA)
-  expect_identical(portion_I(septic_patients$amox, minimum = nrow(septic_patients) + 1),
+  expect_identical(suppressWarnings(portion_I(septic_patients$amox, minimum = nrow(septic_patients) + 1)),
                    NA)
-  expect_identical(portion_S(septic_patients$amox, minimum = nrow(septic_patients) + 1),
+  expect_identical(suppressWarnings(portion_S(septic_patients$amox, minimum = nrow(septic_patients) + 1)),
                    NA)
 
   # warning for speed loss
@@ -83,16 +87,16 @@ test_that("portions works", {
 })
 
 test_that("old rsi works", {
-  # amox resistance in `septic_patients` should be around 66.33%
-  expect_equal(suppressWarnings(rsi(septic_patients$amox)), 0.665, tolerance = 0.0001)
-  expect_equal(suppressWarnings(rsi(septic_patients$amox, interpretation = "S")), 1 - 0.665, tolerance = 0.0001)
+  # amox resistance in `septic_patients` should be around 58.53%
+  expect_equal(suppressWarnings(rsi(septic_patients$amox)), 0.5853, tolerance = 0.0001)
+  expect_equal(suppressWarnings(rsi(septic_patients$amox, interpretation = "S")), 1 - 0.5853, tolerance = 0.0001)
 
   # pita+genta susceptibility around 98.09%
   expect_equal(suppressWarnings(rsi(septic_patients$pita,
                                     septic_patients$gent,
                                     interpretation = "S",
                                     info = TRUE)),
-               0.9540412,
+               0.9498886,
                tolerance = 0.0001)
 
   # count of cases
@@ -117,6 +121,12 @@ test_that("old rsi works", {
       septic_patients$amox %>% portion_I(),
       septic_patients$amox %>% portion_R())
   )
+  expect_equal(
+    septic_patients %>% select(amox) %>% portion_df(combine_IR = TRUE) %>% pull(Value),
+    c(septic_patients$amox %>% portion_S(),
+      septic_patients$amox %>% portion_IR())
+  )
+
 
 })
 
